/*
 * Copyright (C) 2004 ruby-ecore team (see AUTHORS)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <ruby.h>

#include <Ecore.h>

#define __RB_ECORE_C
#include "rb_ecore.h"
#include "rb_timer.h"
#include "rb_animator.h"
#include "rb_idler.h"
#include "rb_idle_enterer.h"
#include "rb_event_handler.h"
#include "rb_fd_handler.h"

VALUE mEcore;

/*
 * call-seq:
 *  Ecore.main_loop_begin
 *
 * Starts the Ecore main loop.
 */
static VALUE m_main_loop_begin (VALUE self)
{
	ecore_main_loop_begin ();

	return Qnil;
}

/*
 * call-seq:
 *  Ecore.main_loop_iterate
 *
 * Run one iteration of the Ecore main loop.
 */
static VALUE m_main_loop_iterate (VALUE self)
{
	ecore_main_loop_iterate ();

	return Qnil;
}

/*
 * call-seq:
 *  Ecore.main_loop_quit
 *
 * Stops the Ecore main loop.
 */
static VALUE m_main_loop_quit (VALUE self)
{
	ecore_main_loop_quit ();

	return Qnil;
}

static VALUE m_time_get (VALUE self)
{
	return rb_float_new (ecore_time_get ());
}

#if 0
static VALUE c_ev_exe_exit_init (VALUE self, VALUE event)
{
	VALUE c = CLASS_OF (self);
	Ecore_Event_Exe_Exit *e = (void *) event;

	rb_define_attr (c, "pid", 1, 0);
	rb_define_attr (c, "exit_code", 1, 0);
	rb_define_attr (c, "exit_signal", 1, 0);
	rb_define_attr (c, "exited", 1, 0);
	rb_define_attr (c, "signalled", 1, 0);

	rb_iv_set (self, "@pid", INT2FIX (e->pid));
	rb_iv_set (self, "@exit_code", INT2FIX (e->exit_code));
	rb_iv_set (self, "@exit_signal", INT2FIX (e->exit_signal));
	rb_iv_set (self, "@exited", e->exited ? Qtrue : Qfalse);
	rb_iv_set (self, "@signalled", e->signalled ? Qtrue : Qfalse);

	return self;
}
#endif

static VALUE c_ev_sig_user_init (VALUE self, VALUE event)
{
	Ecore_Event_Signal_User *e = (void *) event;

	rb_define_attr (CLASS_OF (self), "number", 1, 0);

	rb_iv_set (self, "@number", INT2FIX (e->number));

	return self;
}

static VALUE c_ev_sig_exit_init (VALUE self, VALUE event)
{
	VALUE c = CLASS_OF (self);
	Ecore_Event_Signal_Exit *e = (void *) event;

	rb_define_attr (c, "interrupt", 1, 0);
	rb_define_attr (c, "quit", 1, 0);
	rb_define_attr (c, "terminate", 1, 0);

	rb_iv_set (self, "@interrupt", e->interrupt ? Qtrue : Qfalse);
	rb_iv_set (self, "@quit", e->quit ? Qtrue : Qfalse);
	rb_iv_set (self, "@terminate", e->terminate ? Qtrue : Qfalse);

	return self;
}

static VALUE c_ev_sig_rt_init (VALUE self, VALUE event)
{
	Ecore_Event_Signal_Realtime *e = (void *) event;

	rb_define_attr (CLASS_OF (self), "number", 1, 0);

	rb_iv_set (self, "@number", INT2FIX (e->num));

	return self;
}

static void at_exit ()
{
	ecore_shutdown ();
}

void Init_ecore (void)
{
	VALUE c;

	ecore_init ();

	atexit (at_exit);

	mEcore = rb_define_module ("Ecore");

	rb_define_module_function (mEcore, "main_loop_begin",
	                           m_main_loop_begin, 0);
	rb_define_module_function (mEcore, "main_loop_iterate",
	                           m_main_loop_iterate, 0);
	rb_define_module_function (mEcore, "main_loop_quit",
	                           m_main_loop_quit, 0);
	rb_define_module_function (mEcore, "time", m_time_get, 0);

	Init_Timer ();
	Init_Animator ();
	Init_Idler ();
	Init_IdleEnterer ();
	Init_EventHandler ();
	Init_FdHandler ();

	/* SIGNAL_HUP */
	ADD_EVENT (mEcore, ECORE_EVENT_SIGNAL_HUP,
	           "SignalHupEvent", c);

	/* SIGNAL_POWER */
	ADD_EVENT (mEcore, ECORE_EVENT_SIGNAL_POWER,
	           "SignalPowerEvent", c);

#if 0
	/* EXE_EXIT */
	ADD_EVENT (mEcore, ECORE_EVENT_EXE_EXIT, "ExeExitEvent", c);
	rb_define_private_method (c, "initialize", c_ev_exe_exit_init, 1);
#endif

	/* SIGNAL_USER */
	ADD_EVENT (mEcore, ECORE_EVENT_SIGNAL_USER,
	           "SignalUserEvent", c);
	rb_define_private_method (c, "initialize", c_ev_sig_user_init, 1);

	/* SIGNAL_EXIT */
	ADD_EVENT (mEcore, ECORE_EVENT_SIGNAL_EXIT,
	           "SignalExitEvent", c);
	rb_define_private_method (c, "initialize", c_ev_sig_exit_init, 1);

	/* SIGNAL_REALTIME */
	ADD_EVENT (mEcore, ECORE_EVENT_SIGNAL_REALTIME,
	           "SignalRealtimeEvent", c);
	rb_define_private_method (c, "initialize", c_ev_sig_rt_init, 1);
}

