/*
 * Copyright (C) 2004 ruby-ecore team (see AUTHORS)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <ruby.h>

#include <Ecore.h>
#include <Ecore_Evas.h>

#include "../ecore/rb_ecore.h"
#include "rb_ecore_evas_main.h"
#include "rb_ecore_evas.h"

static void c_free (RbEcoreEvas *ee)
{
	c_ecore_evas_free (ee, true);
}

static VALUE c_alloc (VALUE klass)
{
	RbEcoreEvas *ee = NULL;

	ecore_init ();
	ecore_evas_init ();

	return Data_Make_Struct (klass, RbEcoreEvas,
	                         c_ecore_evas_mark, c_free, ee);
}

/*
 * call-seq:
 *  Ecore::Evas::Fb.new([display, rotation, w, h]) => fb
 *
 * Creates an Ecore::Evas::Fb object.
 */
static VALUE c_init (int argc, VALUE *argv, VALUE self)
{
	VALUE disp, rot, w, h;
	char *cdisp = NULL;
	int irot = 0, iw = 0, ih = 0;

	GET_OBJ (self, RbEcoreEvas, ee);

	rb_scan_args (argc, argv, "04", &disp, &rot, &w, &h);

	if (!NIL_P (disp)) {
		Check_Type (disp, T_STRING);
		cdisp = StringValuePtr (disp);
	}

	if (!NIL_P (rot)) {
		Check_Type (rot, T_FIXNUM);
		irot = FIX2INT (rot);
	}

	if (!NIL_P (w)) {
		Check_Type (w, T_FIXNUM);
		iw = FIX2INT (w);
	}

	if (!NIL_P (h)) {
		Check_Type (h, T_FIXNUM);
		ih = FIX2INT (h);
	}

	ee->real = ecore_evas_fb_new (cdisp, irot, iw, ih);

	rb_call_super (argc, argv);

	return self;
}

void Init_Fb (void)
{
	VALUE c = rb_define_class_under (mEvas, "Fb", cEcoreEvas);

	rb_define_alloc_func (c, c_alloc);
	rb_define_method (c, "initialize", c_init, -1);
}
